function tsgqc_GUI
% tsgqc_GUI
%
% TSG (Thermosalinograph) Quality Control software
%
% $Id$
%

%% COPYRIGHT & LICENSE
%  Copyright 2007 - IRD US191, all rights reserved.
%
%  This file is part of tsgqc_GUI.
%
%    Datagui is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    tsgqc_GUI is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with Datagui; if not, write to the Free Software
%    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


%%  Initialization tasks
%   ********************

% Clear Command Window display, giving up a "clean screen."
% ---------------------------------------------------------
clc;

% Define global variable VERSION number
% -------------------------------------
global VERSION

VERSION = '0.2a';

% reset userdata property of root Matlab object if it is not empty
% ----------------------------------------------------------------
 if ~isempty(get(0,'Userdata'))
   set(0, 'userdata', []);
 end
 
% Find program directory.
% functions (icons) should be store at a lower level
% add directories to Matlab search path, works on UNIX
% and Windows host
% ---------------------------------------------------
tsgqcname = mfilename;
fulltsgqcname = mfilename('fullpath');
DEFAULT_PATH_FILE = strrep(fulltsgqcname, tsgqcname, '') ;
p = [pathsep,...
  DEFAULT_PATH_FILE,[filesep 'tsg_util' pathsep],...
  DEFAULT_PATH_FILE,[filesep 'tsg_data' pathsep],...
  DEFAULT_PATH_FILE,[filesep 'tsg_io' pathsep]
  ];
addpath( p, '-END' );
rehash;

% get screen dimensions (pixels)
% ------------------------------
set(0,'Units','pixels');
screenSize = get(0,'ScreenSize');

% define default font size based on screen resolution
% ---------------------------------------------------
if screenSize(3) <= 1024
  tsg.fontSize = 9;
end

% Screen limits for the GUI
% -------------------------
set(0,'Units','normalized');
guiLimits = get(0,'ScreenSize');
guiLimits(1) = guiLimits(1) + 0.01;
guiLimits(2) = guiLimits(2) + 0.05;
guiLimits(3) = guiLimits(3) - 0.02;
guiLimits(4) = guiLimits(4) - 0.15;


%% Main GUI
% Create and then hide the GUI as it is being constructed.
% --------------------------------------------------------
hMainFig = figure(...
  'Name', 'TSG Validation', ...
  'NumberTitle', 'off', ...
  'Resize', 'on', ...
  'Menubar','none', ...
  'Toolbar', 'none', ...
  'UserData', 'ButtonMotionOff', ...
  'WindowButtonMotionFcn', @MouseMotion, ...
  'CloseRequestFcn', @QuitMenuCallback,...
  'HandleVisibility','callback',...
  'Visible','on',...
  'Tag','TAG_TSG-QC_GUI',...
  'Units', 'normalized',...
  'Position',guiLimits, ...
  'Color', get( 0, 'DefaultUIControlBackgroundColor' ));

% Initialize tsg structure
% ------------------------
%tsg_initialisation(hMainFig)

%% Initialize tsg structure with tsg_preference function
% -----------------------------------------------------
tsg_preferences(hMainFig, tsgqcname);

% get root struct from root object
% --------------------------------
root = get(0,'Userdata');

% Retrieve named application data
% -------------------------------
tsg = getappdata( hMainFig, 'tsg_data');

%%  Menu File
%   -----------------------------------------------------------------------
hFileMenu = uimenu(...
  'Parent', hMainFig,...
  'HandleVisibility','callback',...
  'Label', 'File');
hOpenMenu = uimenu(...
  'Parent', hFileMenu,...
  'Label','Open',...
  'Accelerator','O',...
  'HandleVisibility','callback',...
  'UserData', 'off',...
  'Callback', @OpenMenuCallback);
hSaveMenu = uimenu(...
  'Parent', hFileMenu,...
  'Label','Save',...
  'Accelerator','S',...
  'Enable', 'off',...
  'UserData', 'off',...
  'HandleVisibility','callback',...
  'Callback',@SaveMenuCallback);
hQuitMenu = uimenu(...
  'Parent',hFileMenu,...
  'Label','Quit',...
  'Separator','on',...
  'Accelerator','Q',...
  'HandleVisibility','callback',...
  'Callback',@QuitMenuCallback);

%%  Menu Edit with Undo/Redo submenu 
% ---------------------------------
hEditMenu = uimenu(hMainFig,'Label','Edit');
uimenu(hEditMenu,'Label','Undo',...
  'Accelerator','Z',...
  'Tag','UIMENU_UNDO',...
  'Enable', 'off',...
  'Callback',@UndoMenuCallback);
uimenu(hEditMenu,'Label','Redo',...
  'Accelerator','R',...
  'Tag','UIMENU_REDO',...
  'Enable', 'off',...
  'Callback',@RedoMenuCallback);

%%  Menu Climatology with Annual/Seasonal/Monthly submenu
% --------------------------------------------------------

% define structure s stored in userdata
% -------------------------------------
s = struct('type', 'annual', 'time', 1);

hClimatoMenu = uimenu(hMainFig,'Label','Climatology', ...
  'Tag', 'TAG_UIMENU_CLIMATO_MAIN', ...
  'UserData', s);

% by defautl at startup, select climatology to annual
% ---------------------------------------------------
uimenu(hClimatoMenu,'Label','Annual',...
  'Checked','on',...
  'Tag','TAG_UIMENU_CLIMATO_ANNUAL',...
  'Enable', 'on',...
  'Callback', {@ClimatoSelectMenuCallback, 'annual', 1});

% declare top-level seasonal menu
% -------------------------------
hClimatoSeasonalMenu = uimenu(hClimatoMenu,'Label','Seasonal');

% declare label for seasonal submenu
% ----------------------------------
label = {'jan-feb-mar','apr-may-jun','jul-aug-sept','oct-nov-dec'};

% loop over the four submenu
% --------------------------
for i=1:4
uimenu(hClimatoSeasonalMenu,'Label',label{i},...
  'Checked','off',...
  'Tag',['TAG_UIMENU_CLIMATO_SEASONAL_' i],...
  'Enable', 'on',...
  'Callback',{@ClimatoSelectMenuCallback, 'seasonal', i});
end

% declare top-level monthly menu
% -------------------------------
hClimatoMonthlyMenu = uimenu(hClimatoMenu,'Label','Monthly');

% declare label for monthly submenu
% ----------------------------------
label = {'january','february','march','april','may','june','july','august',...
         'september','october','november','december'};

% loop over the four submenu
% --------------------------
for i=1:12
uimenu(hClimatoMonthlyMenu,'Label',label{i},...
  'Checked','off',...
  'Tag',['TAG_UIMENU_CLIMATO_MONTHLY_' i],...
  'Enable', 'on',...
  'Callback',{@ClimatoSelectMenuCallback, 'monthly', i});
end


%%  Menu Option with Preferences submenu
% --------------------------------------------------------
hOptionMenu = uimenu(hMainFig,'Label','Option');
uimenu(hOptionMenu,'Label','Preferences',...
  'Checked', root.preference.autoload,...
  'Tag','TAG_UIMENU_OPTION_PREFERENCES',...
  'Enable', 'on',...
  'Callback', {@PreferencesMenuCallback});

%%  Toolbar
%   -----------------------------------------------------------------------
hToolbar       =   uitoolbar(...   % Toolbar for Open and Print buttons
  'Parent',hMainFig, ...
  'HandleVisibility','on');
hOpenPushtool  =   uipushtool(...   % Opendoc toolbar button
  'Parent',hToolbar,...
  'TooltipString','Open file',...
  'CData', iconRead(fullfile(matlabroot, ...
  '/toolbox/matlab/icons/opendoc.mat')),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_OPEN',...
  'UserData', 'off',...
  'Enable', 'on',...
  'ClickedCallback', @OpenMenuCallback);
hSavePushtool  =   uipushtool(...   % Open Save toolbar button
  'Parent',hToolbar,...
  'TooltipString','Save NetCDF file',...
  'CData',iconRead( ...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'savedoc.mat']),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_SAVE',...
  'Enable', 'off',...
  'ClickedCallback', @SaveMenuCallback);
hZoomToggletool  =   uitoggletool(...   % Open Zoom toolbar button
  'Parent',hToolbar,...
  'Separator', 'on', ...
  'TooltipString','Zoom',...
  'CData', iconRead(fullfile(matlabroot, ...
  '/toolbox/matlab/icons/zoom.mat')),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_ZOOM',...
  'Enable', 'off',...
  'OffCallback', @Zoom_OffMenuCallback,...
  'ONCallback',  @Zoom_OnMenuCallback);
hPanToggletool  =   uitoggletool(...   % Open Pan toolbar button
  'Parent',hToolbar,...
  'TooltipString','Pan',...
  'CData',iconRead(fullfile(matlabroot, ...
  '/toolbox/matlab/icons/pan.mat')),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_PAN',...
  'Enable', 'off',...
  'OffCallback', @Pan_OffMenuCallback,...
  'OnCallback',  @Pan_OnMenuCallback);
hQCToggletool  =   uitoggletool(...   % Open QC toolbar button
  'Parent',hToolbar,...
  'TooltipString','Validation codes',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'qcicon.mat']),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_QC',...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @QC_OffMenuCallback,...
  'OnCallback',  @QC_OnMenuCallback);
hMapToggletool  =   uitoggletool(...   % Open Map toolbar button
  'Parent',hToolbar,...
  'TooltipString','Map and ship track',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'mapicon.mat']),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_MAP',...
  'UserData', 'off', ...
  'Enable', 'off',...
  'OffCallback', @Map_OffMenuCallback,...
  'OnCallback',  @Map_OnMenuCallback);
hClimToggletool  =   uitoggletool(...   % Open Climatology toolbar button
  'Parent',hToolbar,...
  'TooltipString','Climatology',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'climicon.mat']),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_CLIM',...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @Clim_OffMenuCallback,...
  'OnCallback',  @Clim_OnMenuCallback);
hBottleToggletool  = uitoggletool(...   % Correction module toolbar button
  'Parent',hToolbar,...
  'TooltipString','Correct the SSS TSG data',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'bottleicon.mat']),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_BOTTLE',...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @Bottle_OffMenuCallback,...
  'OnCallback',  @Bottle_OnMenuCallback);
hStartlimitToggletool  = uitoggletool(...   % Select beginning of time series toolbar button
  'Parent',hToolbar,...
  'TooltipString','Select start time',...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'startlimit.mat']),...
  'HandleVisibility','on', ...
  'Tag', 'CORRECT_STARTTIME', ...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @SelectTime_OffMenuCallback,...
  'OnCallback',  @SelectTime_OnMenuCallback);
hEndlimitToggletool  = uitoggletool(...   % Select end of time series
  'Parent',hToolbar,...
  'TooltipString','Select end time',...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'endlimit.mat']),...
  'HandleVisibility','on', ...
  'Tag', 'CORRECT_ENDTIME', ...
  'UserData', 'off',...
  'Enable', 'off',...
  'OffCallback', @SelectTime_OffMenuCallback,...
  'OnCallback',  @SelectTime_OnMenuCallback);

hHeaderPushtool  = uipushtool(...   % Open headerForm button
  'Parent',hToolbar,...
  'TooltipString','Fill the header form',...
  'Separator', 'on', ...
  'CData',iconRead(...
  [DEFAULT_PATH_FILE 'tsg_icon' filesep 'hdricon.mat']),...
  'HandleVisibility','on', ...
  'Tag','PUSHTOOL_HEADER',...
  'Enable', 'off',...
  'ClickedCallback', @HeaderMenuCallback);

%% Dynamic text area
%   -----------------------------------------------------------------------
%  Dynamic text area that displays the loaded filename, date, position and
%  salinity, temperature
%  ------------------------------------------------------------------------

% Create an uipanel
% -----------------
hInfoPanel = uipanel( ...
  'Parent', hMainFig, ...
  'Units', 'normalized', ...
  'BorderType', 'none',...
  'Visible', 'on', ...
  'Position',[.01, .96, .98, .04]);

% Dynamic text area that displays the date
% ----------------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.01, .25, .04, .6], ...
  'String', 'File:');
hInfoFileText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'No file loaded', ...
  'Position', [.05, .25, .1, .6]);

% Text area that displays the date
% --------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.15, .25, .04, .6], ...
  'String', 'Date:');
hInfoDateText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.20, .25, .13, .6]);

% Text area that displays the latitude
% ------------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.33, .25, .06, .6], ...
  'String', 'Latitude:');
hInfoLatText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.4, .25, .9, .6]);

% Text area that displays the longitude
% -------------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.495, .25, .08, .6], ...
  'String', 'Longitude:');
hInfoLongText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.585, .25, .09, .6]);

% Text area that display salinity and temperature
% -----------------------------------------------
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position', [.68, .25, .05, .6], ...
  'String', 'SSPS:');
hInfoSSPSText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.73, .25, .05, .6]);
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.785, .25, .05, .6], ...
  'String', 'SSJT:');
hInfoSSJTText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.835, .25, .05, .6]);
uicontrol(...
  'Parent', hInfoPanel, ...
  'Units', 'normalized', ...
  'Style', 'Text', ...
  'Fontsize', tsg.fontSize, ...
  'Fontweight', 'bold', ...
  'HorizontalAlignment', 'left', ...
  'Position',[.89, .25, .05, .6], ...
  'String', 'SSTP:');
hInfoSSTPText = uicontrol(...
  'Parent', hInfoPanel, ...
  'Style', 'text', ...
  'Fontsize', tsg.fontSize, ...
  'Visible','on',...
  'Units', 'normalized',...
  'HorizontalAlignment', 'left', ...
  'String', 'N/A', ...
  'Position', [.94, .25, .05, .6]);

%% Plot axes
%   -----------------------------------------------------------------------
% The axes 1,2 and 3 will be plot in a uipanel
% axe 1 as propertie 'HandleVisibility' set to 'on' to prevent hidden 
% visibility when first plot is empty
%--------------------------------------------------------------------
hPlotsPanel = uipanel( ...
  'Parent', hMainFig, ...
  'Units', 'normalized', ...
  'BorderType', 'etchedin',...
  'Visible', 'on', ...
  'Position',[0.15, 0.0, .85, .95]); 
hPlotAxes(1) = axes(...     % the axes for plotting Salinity
  'Parent', hPlotsPanel, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'Tag', 'TAG_AXES_1', ...
  'HandleVisibility','on', ...  % set to callback after first plot
  'Position',[.05, .64, .93, .35]);
hPlotAxes(2) = axes(...     % the axes for plotting temperature
  'Parent', hPlotsPanel, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'Tag', 'TAG_AXES_2', ...  
  'HandleVisibility','on', ...
  'Position',[.05, .33, .93, .27]);
hPlotAxes(3) = axes(...     % the axes for plotting ship velocity
  'Parent', hPlotsPanel, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'Tag', 'TAG_AXES_3', ...
  'HandleVisibility','on', ...
  'Position',[.05, .02, .93, .27]);

% The map will be plot in a uipanel
hMapPanel = uipanel( ...
  'Parent', hMainFig, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'Position',[0.15, 0, .85, .58]);
hPlotAxes(4) = axes(...     % the axes for plotting ship track map
  'Parent', hMapPanel, ...
  'Units', 'normalized', ...
  'Visible', 'off', ...
  'Tag', 'TAG_AXES_MAP', ...  
  'Color', 'none', ...
  'UserData', 'off', ...
  'HandleVisibility','on', ...
  'Position',[.05, .05, .9, .9]);

%% Quality Control Button Group panel
%   -----------------------------------------------------------------------
hbgQc = uibuttongroup(...
  'Parent', hMainFig, ...
  'Title', 'Validation Codes', ...
  'FontSize', tsg.fontSize-1, 'Fontweight', 'bold', ...
  'tag', 'TAG_QC_DISPLAY_PANEL',...
  'HandleVisibility','on',...
  'Visible', 'on',...
  'BorderType', 'etchedin',...
  'Position', [.0, .71, .15, .25]);

% Initialize CallBack for button group properties
% ----------------------------------------------- 
set(hbgQc,'SelectionChangeFcn',@RadiobuttonQc);

%% Context Menu and RadioButtons for Quality control
%   -----------------------------------------------------------------------
hQcCmenu = uicontextmenu(...
  'Parent', hMainFig, ...
  'HandleVisibility','on' );

% get list of keys from hashtable tsg.qc.hash, defined inside
% tsg_initialisation.m
% -----------------------------------------------------------
qc_list = get(tsg.qc.hash);

% internal loop count with state to on
% ------------------------------------
count = 0;

% iterate (loop) on each key store inside hastable
% ------------------------------------------------
for i=1:numel(qc_list)
  
  % get key and some values in hashtable
  % ------------------------------------
  key   = qc_list{i};
  label = get(tsg.qc.hash, key, 'label');
  color = get(tsg.qc.hash, key, 'color');
  state = get(tsg.qc.hash, key, 'state');
  
  % construct context menu with only code set to 'on' (valid)
  % ---------------------------------------------------------
  if strcmp( state, 'on')
    
    % add menu to hQcCmenu uicontextmenu
    % ----------------------------------
    uimenu(...
      'Parent', hQcCmenu,...
      'HandleVisibility','on', ...
      'Label', label,...
      'ForegroundColor', color,...
      'Callback', {@Qc, key});
    
    % add button QC to hbgQc uibuttongroup
    % ------------------------------------
    uicontrol(...
      'Parent', hbgQc,...
      'Style', 'radiobutton',...
      'Units', 'normalized', ...
      'Fontsize', tsg.fontSize-1, ...
      'HorizontalAlignment', 'left', ...
      'HandleVisibility','on', ...
      'String', label,...
      'Tag', ['TAG_QC_RADIO_' key], ...
      'ForegroundColor', color,...
      'Position', [.01, .85-count*.12, .6, 0.09]);
    
    % add text QC display statistic on hQcPanel
    % -----------------------------------------
    uicontrol(...
      'Parent', hbgQc,...
      'Style', 'text',...
      'Units', 'normalized', ...
      'Fontsize', tsg.fontSize-1, ...
      'HorizontalAlignment', 'right', ...
      'HandleVisibility','on', ...
      'String', 'N/A ',...
      'ForegroundColor', color,...
      'Tag', ['TAG_QC_TEXT_' key],...
      'Position', [.61, .85-count*.12, .37, 0.09]);
    
    % increment count
    % ---------------
    count = count + 1;
  end
end

%% uiButtonGroup set to choose the Correction method
% -------------------------------------------------------------------------

% Create the button group
hbgCorMethod = uibuttongroup( ...
  'Parent',hMainFig, ...
  'Title','Correction Method', ...
  'Units', 'normalized', ...
  'FontSize',tsg.fontSize-1, 'Fontweight', 'bold',...
  'Position',[.0, .3, .15, .18]);

% Create 2 radio buttons in the button group
hrbCorCancel = uicontrol( ...
  'Style','pushbutton', ...
  'Parent',hbgCorMethod, ...
  'Units', 'normalized', ...
  'String','Cancel the adjustment',...
  'FontSize',tsg.fontSize-1,...
  'Tag', 'CORRECT_CANCEL_PUSH', ...
  'Enable', 'off',...
  'pos',[.05 .08 .9 .25],...
  'HandleVisibility','callback', ...
  'Callback', @CorCancelCallback);
hrbCorLinear = uicontrol( ...
  'Style','pushbutton', ...
  'Parent',hbgCorMethod, ...
  'Units', 'normalized', ...
  'String','Linear adjustment',...
  'FontSize',tsg.fontSize-1,...
  'Tag', 'CORRECT_LINEAR_PUSH', ...
  'Enable', 'off',...
  'pos',[.05 .38 .9 .25], ...
  'HandleVisibility','callback', ...
  'Callback', @CorLinearCallback);
hrbCorMedian = uicontrol( ...
  'Style','pushbutton', ...
  'Parent',hbgCorMethod, ...
  'Units', 'normalized', ...
  'String','Running median filter',...
  'FontSize',tsg.fontSize-1,...
  'Tag', 'CORRECT_MEDIAN_PUSH', ...
  'Enable', 'off',...
  'pos',[.05 .68 .9 .25], ...
  'HandleVisibility','callback', ...
  'Callback', @CorMedianCallback);

%% uiPanel for the Date limits used in the Correction module

% Choose the date limits for the correction
% --------------------------------------------------
% Create the uipanel
hpDateLimit = uipanel( ...
  'Parent', hMainFig, ...
  'Title', 'Date Limits', ...
  'Units', 'normalized', ...
  'FontSize', tsg.fontSize-1, 'Fontweight', 'bold', ...
  'Position', [.0, .51, .15, .18]);
%  'BackgroundColor', 'white',...

htDateMin = uicontrol( ...
  'Parent', hpDateLimit, ...
  'Style', 'Text', ...
  'String', 'Min : yyyy-mm-dd hh:mm:ss', ...
  'HorizontalAlignment', 'left', ...
  'Units', 'normalized', ...
  'FontSize', tsg.fontSize-1, ...
  'Position',[.01 .8 .95 .15]);
hetDateMin = uicontrol( ...
  'Parent', hpDateLimit, ...
  'Style', 'edit', ...
  'Units', 'normalized', ...
  'BackgroundColor', 'white',...
  'Enable', 'off', ...
  'FontSize', tsg.fontSize, ...
  'Tag', 'CORRECT_DATEMIN_EDIT',...
  'Position',[.01 .6 .95 .17]);

htDateMax = uicontrol( ...
  'Parent', hpDateLimit, ...
  'Style', 'Text', ...
  'String', 'Max : yyyy-mm-dd hh:mm:ss', ...
  'HorizontalAlignment', 'left', ...
  'Units', 'normalized', ...
  'FontSize', tsg.fontSize-1, ...
  'Position',[.01 .35 .95 .15]);
hetDateMax = uicontrol( ...
  'Parent', hpDateLimit, ...
  'Style', 'edit', ...
  'Units', 'normalized', ...
  'BackgroundColor', 'white',...
  'FontSize', tsg.fontSize, ...
  'Tag', 'CORRECT_DATEMAX_EDIT',...
  'Enable', 'off', ...
  'Position',[.01 .15 .95 .17]);



%% test if user preference autoload field is checked (on)
% -------------------------------------------------------
if strcmp(root.preference.autoload, 'on')

  % this code need to be fatorized file/open menu
  % ---------------------------------------------
  
  % A TSG file has been open and read
  % ---------------------------------
  set( hOpenMenu, 'UserData', 'on' );

  % Show handles marked as hidden with HandleVisibility property set to
  % callback or off
  % -------------------------------------------------------------------
  set(0, 'ShowHiddenHandles', 'on');
  
  % enable toolbar menu pushtool
  % ----------------------------
  hdl_pushtool = findobj('-regexp','Tag', 'PUSHTOOL_');
  set(hdl_pushtool, 'Enable', 'on');

  % enable Save menu
  % ----------------
  set(hSaveMenu, 'Enable', 'on');

  % update the filename display
  % ---------------------------
  set( hInfoFileText, 'String', tsg.file.name);

  % update some fields in tsg structure
  % -----------------------------------
  updateTsgStruct(hMainFig);

  % Running average of TSG time series
  % ----------------------------------
  tsg_moveaverage(hMainFig);

  % Get application data before drawing
  % -----------------------------------
  tsg = getappdata( hMainFig, 'tsg_data');
  
  % yes, plot the Salinity, temperature and velocity
  % ------------------------------------------------
  plot_Tsg( hMainFig, hPlotAxes, 1, tsg.DAYD, tsg.ssps_smooth, [],...
            'SSPS','k','none','.',1);

  plot_Tsg( hMainFig, hPlotAxes, 1, tsg.DAYD, tsg.SSPS, tsg.SSPS_QC,...
            'SSPS','','none','.',1);
  plot_Tsg( hMainFig, hPlotAxes, 2, tsg.DAYD, tsg.SSJT,[],...
            'SSJT','k','none','.',1);
  plot_Tsg( hMainFig, hPlotAxes, 3, tsg.DAYD, tsg.SPDC,[],...
            'SPDC','k','none','.',1);
  axesCommonProp( hPlotAxes );
  
  if ~isempty( tsg.SSPS_WS ) || ~isempty( tsg.SSPS_EXT )
    tsg_mergesample( hMainFig );
    plot_Sample( hMainFig, hPlotAxes, 1, 'SSPS', [] );
  end
  
  plot_map( hMainFig, hPlotAxes);

  % The callback to detect the mouse motion can be set to on
  % --------------------------------------------------------
  set( hMainFig, 'UserData', 'ButtonMotionOn');

  % Hide handles marked as hidden with HandleVisibility property set to
  % callback or off
  % -------------------------------------------------------------------
  set(0, 'ShowHiddenHandles', 'off');
  
end


%% *************************** CALLBACKS **********************************

%% OpenMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the Open menu item is selected
  %----------------------------------------------------------------------
  function OpenMenuCallback(hObject, eventdata)
 
    % Pointer set to watch during reading and plotting
    % ------------------------------------------------
    set( hMainFig, 'Pointer', 'watch' );

    % Open standard dialog box for retrieving files
    % ---------------------------------------------
    [fileName, pathname, filterIndex] = uigetfile( ...
      {'*.txt';'*.xml';'*.nc';'*.lbv';'*.btl'; '*.arg'}, 'Pick a file');

    error1 = -1;
    error2 = -1;

    % flushes the event queue and updates the closed uigetfile window
    % ---------------------------------------------------------------
    drawnow;
    
    % if the user clicks the Cancel button or closes the dialog window, 
    % FileName and PathName are set to 0.
    % -----------------------------------------------------------------
    if ~isequal(fileName, 0)
      
      % construct valid and full file path
      % -----------------------------------
      fullFileName = [pathname filesep fileName];

      % Read the data
      % -------------
      switch filterIndex

        case 1
          error1 = readTsgDataTxt(     hMainFig, fullFileName );
        case 2
          error1 = readTsgDataXML(     hMainFig, fullFileName );
        case 3
          error1 = readTsgDataNetCDF(  hMainFig, fullFileName );       
          %# a modifier
          if error1 == 1
            error2 = error1;
          end
        case 4
          error1 = readTsgDataLabview( hMainFig, fullFileName ); 
          if error1 ~= -1
            automaticQC( hMainFig );
          end
        case 5
          error2 = readBucketData(     hMainFig, fullFileName );
        case 6
          error2 = readCoriolisData(   hMainFig, fullFileName );
        otherwise
          return;

      end

      % A TSG file has been read. Plot the data.
      % ----------------------------------------
      if error1 ~= -1

        % A TSG file has been open and read
        % ---------------------------------
        set( hOpenMenu, 'UserData', 'on' );
        
        % enable toolbar menu pushtool
        % ----------------------------
        hdl_pushtool = findobj('-regexp','Tag', 'PUSHTOOL_');
        set(hdl_pushtool, 'Enable', 'on');
        
        % enable Save menu
        % ----------------
        set(hSaveMenu, 'Enable', 'on');

        % update the filename display
        % ---------------------------
        set( hInfoFileText, 'String', fileName);
        
        % update some fields in tsg structure
        % -----------------------------------
        updateTsgStruct(hMainFig);
        
        % Running average of TSG time series
        % ----------------------------------
        tsg_moveaverage(hMainFig);

        % Get application data
        % --------------------
        tsg = getappdata( hMainFig, 'tsg_data');

        % The callback to detect the mouse motion can be set to on
        % --------------------------------------------------------
        set( hMainFig, 'UserData', 'ButtonMotionOn');

        % Make the Salinity, temperature and velocity plot
        % ------------------------------------------------
        plot_Tsg( hMainFig, hPlotAxes, 1, tsg.DAYD, tsg.ssps_smooth, [],...
                  'SSPS','k','-','*',2);

        plot_Tsg( hMainFig, hPlotAxes, 1, tsg.DAYD, tsg.SSPS, tsg.SSPS_QC,...
                  'SSPS','','none','*',2);
        plot_Tsg( hMainFig, hPlotAxes, 2, tsg.DAYD, tsg.SSJT,[],...
                  'SSJT','k','none','*',2);
        plot_Tsg( hMainFig, hPlotAxes, 3, tsg.DAYD, tsg.SPDC,[],...
                  'SPDC','k','none','*',2);
        axesCommonProp( hPlotAxes );
        
        % Update QC statistics
        % --------------------
        display_QC( hMainFig, hPlotAxes)

      end
    end

    % Merge bucket and external samples
    % ---------------------------------
    if ~isempty( tsg.SSPS_WS ) || ~isempty( tsg.SSPS_EXT )
      tsg_mergesample( hMainFig );
    end
    
    % Plot Salinity bucket 
    % --------------------
    if error2 ~= -1
      plot_Sample( hMainFig, hPlotAxes, 1, 'SSPS', [] );
    end
    
    % Pointer reset to arrow
    % ----------------------
    set( hMainFig, 'Pointer', 'arrow' );

  end

%% Zoom_OffMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the zoom toggle toolbar is released
  %----------------------------------------------------------------------
  function Zoom_OffMenuCallback(hObject, eventdata)
    
    % turns interactive zooming off
    % -----------------------------
    zoom off;
 
    % Set the right limit and interval to the 3 axes
    % ----------------------------------------------
    for iaxe = 1:3
      set(hPlotAxes(iaxe),'XTickMode','auto')
      datetick(hPlotAxes(iaxe),'x','keeplimits')
    end
    
    % Turns off the automatic adaptation of date ticks
    % ------------------------------------------------
    zoomAdaptiveDateTicks('off');
    
  end

%% Zoom_OnMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the zoom toggle toolbar is released
  %----------------------------------------------------------------------
  function Zoom_OnMenuCallback(hObject, eventdata)
    
    % cursor back to normal
    % ---------------------
    set(hMainFig,'Pointer','arrow');
    
    % returns a zoom mode object for the figure hMainFig handle
    % --------------------------------------------------------
    hZoom = zoom(hMainFig);
    
    % Disallows a zoom operation on the MAP axes objects
    % --------------------------------------------------
    setAllowAxesZoom(hZoom, hPlotAxes(4), false);

    % turns on interactive zooming (same effect than zoom on) but prevent
    % side effect on another figure
    % -------------------------------------------------------------------
    set(hZoom,'enable', 'on');
    %zoom on;
    
    % Turns on the automatic adaptation of date ticks
    % -----------------------------------------------
    zoomAdaptiveDateTicks('on');
 
    % Desactivate some toggle buttons
    % -------------------------------
    set( hQCToggletool,  'state', 'off' );
    set( hPanToggletool, 'state', 'off' );
    set( hStartlimitToggletool, 'State', 'off' );
    set( hEndlimitToggletool,   'State', 'off' );
   
  end

%% Pan_OffMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the pan toggle toolbar is released
  %----------------------------------------------------------------------
  function Pan_OffMenuCallback(hObject, eventdata)
    
    % turns interactive pan off
    % -------------------------
    pan off;
    
    % Set the right limit and interval to the 3 axes
    % ----------------------------------------------
    for iaxe = 1:3
      set(hPlotAxes(iaxe),'XTickMode','auto')
      datetick(hPlotAxes(iaxe),'x','keeplimits')
    end
 
    % Turns off the automatic adaptation of date ticks
    % ------------------------------------------------
    panAdaptiveDateTicks('off');
    
  end

%% Pan_OnMenuCallback
  %----------------------------------------------------------------------
  % Callback function run when the pan toggle toolbar is selected
  %----------------------------------------------------------------------
  function Pan_OnMenuCallback(hObject, eventdata)
    
    % cursor back to normal
    % ---------------------
    set(hMainFig,'Pointer','arrow');
    
    % Returns a pan mode object for the figure handle
    % -----------------------------------------------
    hPan = pan(hMainFig);
    
    % Disallows a pan operation on the MAP axes objects
    % --------------------------------------------------
    setAllowAxesPan(hPan, hPlotAxes(4), false);

    % turns on interactive pan (same effect than pan on) but prevent
    % side effect on another figure
    % --------------------------------------------------------------
    set(hPan,'enable', 'on');
    
    % Turns on the automatic adaptation of date ticks
    % -----------------------------------------------
    panAdaptiveDateTicks('on');
   
    % Desactivate some toggle buttons
    % -------------------------------
    set( hQCToggletool,   'state', 'off' );
    set( hZoomToggletool, 'state', 'off' );
    set( hStartlimitToggletool, 'State', 'off' );
    set( hEndlimitToggletool,   'State', 'off' );
    
  end

%% QC_OffMenuCallback .............................. Quality Control Module
  %----------------------------------------------------------------------
  % Callback function run when the QC toggle tool is released
  %----------------------------------------------------------------------
  function QC_OffMenuCallback(gcbo, eventdata)
    
    % Desactive right clic menu on first axes (salinity)
    % ---------------------------------------------------
    set(hPlotAxes(1),'UIContextMenu', []);
    
    % Desactive clic mouse menu (zoom) on first axes (salinity)
    % ----------------------------------------------------------
    set(hPlotAxes(1),'ButtonDownFcn', []);
      
%     % No selection on uibuttongroup QC
%     % --------------------------------
%     set(hbgQc,'SelectedObject',[]);  
       
    % cursor back to normal
    % ---------------------
    set(hMainFig,'Pointer','arrow');
    
  end

%% QC_OnMenuCallback ............................... Quality Control Module
  %----------------------------------------------------------------------
  % Callback function run when the QC toggle tool is pressed
  %----------------------------------------------------------------------
  function QC_OnMenuCallback(gcbo, eventdata)
    
    % Desactivate Zoom and Pan functions.
    % ----------------------------------
    set( hZoomToggletool,       'state', 'off' );
    set( hPanToggletool,        'state', 'off' );
    set( hStartlimitToggletool, 'State', 'off' );
    set( hEndlimitToggletool,   'State', 'off' );

    % Activate right clic context menu on first axes (salinity)
    % ---------------------------------------------------------
    set(hPlotAxes(1),'UIContextMenu', hQcCmenu);

    % Activate clic mouse menu on first axes (salinity) for next rbbox
    % ----------------------------------------------------------------
    set(hPlotAxes(1),'ButtonDownFcn', @QC_SelectCallback);
    
    % change cursor to crosshair aspect
    % ---------------------------------
    set( hMainFig, 'Pointer', 'crosshair');
    
    % -------------------------------------------------------------
    % nested function on mouse clic when QC toggle tool is selected
    % -------------------------------------------------------------
    function QC_SelectCallback(gcbo, eventdata)
      
      % disable ButtonMotion on main fig during select
      % prevent drawing to map
      % ----------------------------------------------
      set( hMainFig, 'WindowButtonMotionFcn', []);

      % Retrieve named application data
      % -------------------------------
      sample = getappdata( hMainFig, 'sample');
      tsg    = getappdata( hMainFig, 'tsg_data');

      % Selection of the data within the figure
      % ---------------------------------------
      point1    = get(gca,'CurrentPoint');    % button down detected
      finalRect = rbbox;                      % return figure units
      point2    = get(gca,'CurrentPoint');    % button up detected

      point1 = point1(1,1:2);                 % extract x and y
      point2 = point2(1,1:2);

      p1 = min(point1,point2);
      p2 = max(point1,point2);                % calculate locations

      % The following code is only executed if the left mouse button is clicked.
      % If the right mouse button is clicked, this code must not be
      % executed as the variable 'ind' will be emptied. And this variable
      % is used when the context menu (callback 'Qc') is called
      % -----------------------------------------------------------------
      if ~strcmp( get(gcf, 'SelectionType'), 'alt')

        % The QC is applied either on TSG data either on Sample data.
        % it depends if the Botte Toggle button has been cliked on
        % -----------------------------------------------------------
        if strcmp( get(hBottleToggletool, 'state'), 'on')

          % get index on selected zone
          % --------------------------
          ind = find(sample.DAYD     > p1(1,1) & sample.DAYD     < p2(1,1) & ...
                     sample.SSPS_DIF > p1(1,2) & sample.SSPS_DIF < p2(1,2));

          % Keep the information on the indices of the selected zone
          % --------------------------------------------------------
          tsg.rbboxind = ind .* ones(size(ind));
          
          % Modifiy the QC
          % --------------
          sample.SSPS_QC(ind) = tsg.qc.active.Code;

          % Save the modifications
          % ----------------------
          setappdata( hMainFig, 'tsg_data', tsg);
          setappdata( hMainFig, 'sample', sample);

          % plot selected data with selected code
          % --------------------------------------
          erase_Line( hPlotAxes, 1 );
          erase_Line( hPlotAxes, 2 );
          plot_Sample( hMainFig, hPlotAxes, 1, 'SSPS_DIF', [] );
          plot_Tsg( hMainFig, hPlotAxes, 2, tsg.DAYD, tsg.SSPS, tsg.SSPS_QC,...
                    'SSPS','','none','.',1);
          plot_Sample( hMainFig, hPlotAxes, 2, 'SSPS', [] );

        else

          % get index on selected zone
          % ---------------------------
          ind = find(tsg.DAYD > p1(1,1) & tsg.DAYD < p2(1,1) & ...
                     tsg.SSPS > p1(1,2) & tsg.SSPS < p2(1,2));

          % Keep the information on the indices of the selected zone
          % --------------------------------------------------------
          tsg.rbboxind = ind .* ones(size(ind));
          
          % Modifiy the QC
          % --------------
          tsg.SSPS_QC(ind) = tsg.qc.active.Code;

          % put last SSPS_QC in queue (Undo/Redo)
          % -------------------------------------
          tsg.queue = push( tsg.queue, tsg.SSPS_QC);

          % store QC in queue object, for undo
          % ----------------------------------
          tsg.queue = push(tsg.queue, tsg.SSPS_QC);

          % Save the modifications
          % ----------------------
          setappdata( hMainFig, 'tsg_data', tsg);

          % plot selected data with selected code
          % --------------------------------------
          erase_Line( hPlotAxes, 1 );
          plot_Tsg( hMainFig, hPlotAxes, 1, tsg.DAYD, tsg.SSPS, tsg.SSPS_QC,...
                    'SSPS','','none','.',1);
          plot_Sample( hMainFig, hPlotAxes, 1, 'SSPS', [] );

          % refresh QC statistic panel
          % --------------------------
          display_QC( hMainFig, hPlotAxes );

          % enable undo menu
          % ----------------
          set(findobj('tag','UIMENU_UNDO'),'enable','on');
        end

        % As soon as a modification took place the data should be saved
        % -------------------------------------------------------------
        set( hSaveMenu, 'UserData', 'on' );

      end

      % enable ButtonMotion on main fig after select QC area
      % ----------------------------------------------------
      set( hMainFig, 'WindowButtonMotionFcn', @MouseMotion);

    end
  end

%% Qc context menu selected ........................ Quality Control Module
  %-----------------------------------------------------------
  % Callback function run when the QC context menu is selected
  %
  % Context menu used to attribute a QC to the TSG time series
  %
  %-----------------------------------------------------------
  function Qc(hObject, eventdata, key)

    % Retrieve Default Quality Code and Color
    % ---------------------------------------
    tsg    = getappdata( hMainFig, 'tsg_data');
    sample = getappdata( hMainFig, 'sample');

    % get key and some values in hashtable
    % ------------------------------------
    code  = get(tsg.qc.hash, key, 'code');
    color = get(tsg.qc.hash, key, 'color');

    % set active code and color from selected context menu
    % ----------------------------------------------------
    tsg.qc.active.Code  = code;
    tsg.qc.active.Color = color;
    
    % Modify and plot the last selected tsg data
    % ------------------------------------------
    if ~isempty( tsg.rbboxind )

      if strcmp( get(hBottleToggletool, 'state'), 'on')

        sample.SSPS_QC(tsg.rbboxind) = tsg.qc.active.Code;

        % Save tsg.SSPS_QC and tsg.qc.active.Code 
        % in the application data
        % ----------------------------------------
        setappdata( hMainFig, 'sample', sample);
        setappdata( hMainFig, 'tsg_data', tsg );

        % Plot selected data with the new codes
        % --------------------------------------
        erase_Line( hPlotAxes, 1 );
        erase_Line( hPlotAxes, 2 );
        plot_Sample( hMainFig, hPlotAxes, 1, 'SSPS_DIF', [] );
        plot_Tsg( hMainFig, hPlotAxes, 2, tsg.DAYD, tsg.SSPS, tsg.SSPS_QC,...
                 'SSPS','','none','.',1);
        plot_Sample( hMainFig, hPlotAxes, 2, 'SSPS', [] );

      else
        tsg.SSPS_QC(tsg.rbboxind) = tsg.qc.active.Code;

        % Save tsg.SSPS_QC in the application data
        % ----------------------------------------
        setappdata( hMainFig, 'tsg_data', tsg );
        
        % Plot the TSG data with new codes
        % --------------------------------
        erase_Line( hPlotAxes, 1 );
        plot_Tsg( hMainFig, hPlotAxes, 1, tsg.DAYD, tsg.SSPS, tsg.SSPS_QC,...
                 'SSPS','','none','.',1);
        plot_Sample( hMainFig, hPlotAxes, 1, 'SSPS', [] );
      end

    end

    % Update the radio button corresponding to the selected
    % QC context menu (rbg : RadioButtonGroup)
    % -----------------------------------------------------
    radioTag     = ['TAG_QC_RADIO_' key];        % Build the TAG of the CODE
    hbgChildren  = get(hbgQc,'Children');        % Get the handles of the rbg
    hbgTagCell   = get(hbgChildren, 'tag');      % get the TAG of the rbg
    ind          = strcmp(radioTag, hbgTagCell); % Compare the TAG
    hRadioButton = hbgChildren( ind == 1 );      % Get the handle of the button
    set(hbgQc, 'SelectedObject', hRadioButton);  % Make this button active

    setappdata( hMainFig, 'tsg_data', tsg );

  end

%% Radiobutton Quality Control ..................... Quality Control Module
  % ---------------------------------------------------------------
  % Callback to select CODE and COLOR QC from the RadioButton Group
  % ---------------------------------------------------------------
  function RadiobuttonQc(source, eventdata)
        
      % Retrieve Default Quality Code and Color
      % ---------------------------------------
      tsg = getappdata( hMainFig, 'tsg_data');
     
      % Retrieve the key from the active RadioButton
      % --------------------------------------------
      rbTag = get(eventdata.NewValue,'Tag');
      key = strrep(rbTag, 'TAG_QC_RADIO_', '');

      % store his handle to uibuttongroup userdata
      % ------------------------------------------
      set(hbgQc,'Userdata', eventdata.NewValue);
      
      % get key and some values in hashtable
      % ------------------------------------
      code  = get(tsg.qc.hash, key, 'code');
      color = get(tsg.qc.hash, key, 'color');
       
      % set active code and color from selected context menu
      % ----------------------------------------------------
      tsg.qc.active.Code  = code;
      tsg.qc.active.Color = color;

      setappdata( hMainFig, 'tsg_data', tsg );

  end

%% preQcPanCallback .................................Quality Control Module
function preQcPanCallback(obj, evd)
  
  set(hQCToggletool, 'state', 'off' );

end

%% postQcPanCallback ............................... Quality Control Module
function postQcPanCallback(obj, evd)
  
  set(hQCToggletool, 'state', 'on' );
  
  % Set the right limit and interval to the 3 axes
  % ----------------------------------------------
  for iaxe = 1:3
    set(hPlotAxes(iaxe),'XTickMode','auto')
    datetick(hPlotAxes(iaxe),'x','keeplimits')
  end
  
end

%% MouseMotion
  %---------------------------------------------------------------------
  % Callback function run when mouse pointer is moving on temperature plot
  % draw corresponding measurement position on map
  %---------------------------------------------------------------------
  function MouseMotion(hObject, eventdata)

    % Test if the callback can be activated
    % -------------------------------------
    if strcmp( get( hMainFig, 'UserData'), 'ButtonMotionOn')

      % Retrieve named application data
      % -------------------------------
      tsg = getappdata( hMainFig, 'tsg_data');

      % Get current position of cusor and return its coordinates in
      % axes with handle h_axes
      % -----------------------------------------------------------
      a = get(hPlotAxes(1), 'CurrentPoint');
      x = a(2,1);
      y = a(2,2);
      
      % Get the Limits of axes 1
      % ------------------------
      limx = get(hPlotAxes(1), 'XLim');
      limy = get(hPlotAxes(1), 'YLim');

      % Code to Activate the PAN function when QC mode is active
      % A PAN zone is defined in the bottom (10%) of PlotAxes(1)
      % 2 callback are needed : 
      %    1 - one to desactivate QC when Pan is set to on.
      %    2 - one to reactivate QC once the pan has been used.
      % ---------------------------------------------------------
      qcState = get(hQCToggletool,   'state' );
      if strcmp(qcState, 'on' );
        
        % Suppose that Y axes is increasing from the bottom to the top
        % ------------------------------------------------------------
        limy2 = limy(1) + (limy(2)-limy(1)) * 0.10;

        if  x > limx(1) && x < limx(2) &&  y <= limy2 && y >= limy(1)
          hPan = pan(hMainFig);
          set(hPan,'ActionPreCallback',  @preQcPanCallback);
          set(hPan,'ActionPostCallback', @postQcPanCallback);
          set(hPan,'Enable','on');
        else
          pan off
        end
      end
      
      % Dynamically display data in uicontrol
      % -------------------------------------
      %if x > tsg.DAYD(1) && x < tsg.DAYD(end)
      if  x > limx(1) && x < limx(2)

        indCursor = find( tsg.DAYD > x);
        set( hInfoDateText, 'String',...
          datestr(tsg.DAYD(indCursor(1)),'dd/mm/yyyy   HH:MM'));
        set( hInfoLatText,  'String', dd2dm(tsg.LATX(indCursor(1)),0));
        set( hInfoLongText, 'String', dd2dm(tsg.LONX(indCursor(1)),1));
        set( hInfoSSPSText, 'String', tsg.SSPS(indCursor(1)));
        set( hInfoSSJTText, 'String', tsg.SSJT(indCursor(1)));
        if ~isempty(tsg.SSTP)
          set( hInfoSSTPText, 'String', tsg.SSTP(indCursor(1)));
        end

        % Plot the position on the map if this one is active
        % --------------------------------------------------
        if strcmp( get(hMapPanel, 'Visible'), 'on')

          % Select the map axes
          % -------------------
          axes( hPlotAxes(4));

          if isempty( get(hMapPanel, 'UserData'))
            hMarker = m_line( tsg.LONX(indCursor(1)), tsg.LATX(indCursor(1)),...
                              'Marker','o','MarkerSize',5, ...
                              'Color','r', 'MarkerFaceColor','r');
            set(hMapPanel, 'UserData', hMarker)
          else
            delete(get(hMapPanel, 'UserData'));
            hMarker = m_line( ...
                             tsg.LONX(indCursor(1)), tsg.LATX(indCursor(1)),...
                             'Marker','o','MarkerSize',5, ...
                             'Color','r', 'MarkerFaceColor','r');
            set(hMapPanel, 'UserData', hMarker);
          end
          
        end
      end
    end
  end

%% Map_OffMenuCallback
  %---------------------------------------------------------------------
  % Callback function run when the Map tool bar item is unselected
  %---------------------------------------------------------------------
  function Map_OffMenuCallback(hObject, eventdata)
    
    %if ~isempty( get(hMapPanel, 'UserData'))
    %  delete(get(hMapPanel, 'UserData'));
    %end

    %hLines = findobj( hPlotAxes(4), 'Tag', 'TAG_AXES_MAP');
    %hLines = get( hPlotAxes(4), 'Children' );
    %if ~isempty( hLines )
    %  delete(hLines);
    %end
    
   % Make the earth map invisible
    % ----------------------------
    set(hMapPanel, 'Visible', 'off' ); 

  end

%% Map_OnMenuCallback
  %---------------------------------------------------------------------
  % Callback function run when the Map tool bar item is selected
  %---------------------------------------------------------------------
  function Map_OnMenuCallback(hObject, eventdata)
     
    % Desactivate Zoom and Pan toggle buttons
    % may be change in the futur ...
    % ---------------------------------------
    set(hZoomToggletool,  'state', 'off' );
    set(hPanToggletool,  'state', 'off' );

    % Make the earth map visible
    % --------------------------
    set(hMapPanel, 'Visible', 'on' );
    
    erase_Line( hPlotAxes, 4 );

    plot_map( hMainFig, hPlotAxes)
    
  end

%% Bottle_OnMenuCallback ................................ Correction Module
  %---------------------------------------------------------------------
  % Callback function run when the bootle push tool is selected
  %---------------------------------------------------------------------
  function Bottle_OnMenuCallback(hObject, eventdata)
    % Callback function run when the Bottle tool bar item is selected

    % Get the tsg structure
    % ---------------------
    tsg = getappdata(hMainFig, 'tsg_data');
    
    % Desactivate the Climatology button
    % ----------------------------------
    set( hClimToggletool, 'Enable', 'off');
    
    % Switch somme buttons
    % --------------------
    set( hZoomToggletool, 'state', 'off' );
    set( hQCToggletool,   'state', 'off' );
    set( hPanToggletool,  'state', 'off' );
    
    % Activate some toolbar buttons
    % -----------------------------
    hdl_Toggletool = findobj('-regexp','Tag', 'CORRECT_');
    set(hdl_Toggletool, 'Enable', 'on');
    
    % Test if tsg and sample data have been loaded
    % --------------------------------------------
    if ~isempty( tsg.SSPS_WS ) || ~isempty( tsg.SSPS_EXT )
      
      % Compute the sample-TSG differences
      % ----------------------------------
      diffTsgSample( hMainFig );
      
      % Plot in the 3 axes
      % ------------------
      plot_Correction( hMainFig, hPlotAxes );

      % Get the information on time limits of the time series
      % Write them in the uipanel
      % -----------------------------------------------------
      noNaN = tsg.DAYD(~isnan( tsg.DAYD ));
      set( hetDateMin, 'String', datestr(noNaN(1), 31));
      set( hetDateMax, 'String', datestr(noNaN(end), 31)); 

    else

      % Desactivate the corretion module
      % --------------------------------
      set( hBottleToggletool, 'state', 'off' );
      
      msgbox('Sample data not loaded in the program', 'modal');

    end
  end

%% Bottle_OffMenuCallback ............................... Correction module
  %---------------------------------------------------------------------
  % Callback function run when the bootle push tool is selected
  %---------------------------------------------------------------------
  function Bottle_OffMenuCallback(hObject, eventdata)
 
    % Transfert de la structure Sample  la structure 
    % RESTE A ECRIRE
    % -----------------------------------------------
    
    % Activate the Climatology button
    % -------------------------------
    set( hClimToggletool, 'Enable', 'on');
    
    % If necessary toggle off some buttons
    % ------------------------------------
    set( hZoomToggletool, 'state',  'off' );
    set( hQCToggletool,   'state',  'off' );
    set( hPanToggletool,  'state',  'off' );
    
    % Desactivate some toolbar buttons
    % --------------------------------
    hdl_Toggletool = findobj('-regexp','Tag', 'CORRECT_');
    set(hdl_Toggletool, 'Enable', 'off');

    set( hStartlimitToggletool, 'State', 'off' );
    set( hEndlimitToggletool,   'State', 'off' );

    % Desactivate Click Mouse on figure
    % ---------------------------------
    set( hMainFig, 'WindowButtonDownFcn', []);

    % Plot the SSS, SST and Velocity
    % ------------------------------
    erase_Line( hPlotAxes, 1 );
    erase_Line( hPlotAxes, 2 );
    erase_Line( hPlotAxes, 3 );
    
    plot_Tsg( hMainFig, hPlotAxes, 1, tsg.DAYD, tsg.ssps_smooth, [],...
              'SSPS','k','-','*',2);

    plot_Tsg( hMainFig, hPlotAxes, 1, tsg.DAYD, tsg.SSPS, tsg.SSPS_QC,...
              'SSPS','','none','*',2);
    plot_Sample( hMainFig, hPlotAxes, 1, 'SSPS', [] );
    plot_Tsg( hMainFig, hPlotAxes, 2, tsg.DAYD, tsg.SSJT,[],...
              'SSJT','k','none','*',2);
    plot_Tsg( hMainFig, hPlotAxes, 3, tsg.DAYD, tsg.SPDC,[],...
              'SPDC','k','none','*',2);
    
    axesCommonProp( hPlotAxes );
 
  end

%% SelectTime_OnMenuCallback ............................ Correction module
%----------------------------------------------------------------------
  function SelectTime_OnMenuCallback(hObject, eventdata)
    % Callback function run when the ....
            
    % Desactivate Zoom and Pan functions.
    % ----------------------------------
    set( hZoomToggletool, 'state', 'off' );
    set( hQCToggletool,   'state', 'off' );
    set( hPanToggletool,  'state', 'off' );
    
    % Create a pointer to select the time limits
    % ------------------------------------------
    selTimePointer = ones(16)+1;
    selTimePointer(1,:)       = 1; selTimePointer(16,:)      = 1;
    selTimePointer(:,1)       = 1; selTimePointer(:,16)      = 1;
    selTimePointer(1:4,8:9)   = 1; selTimePointer(13:16,8:9) = 1;
    selTimePointer(8:9,1:4)   = 1; selTimePointer(8:9,13:16) = 1;
    selTimePointer(5:12,5:12) = NaN; % Create a transparent region in the center
   
    % Toggle the state of the Qc Togglebutton to 'on' or 'off'
    % ----------------------------------------------------
    if hObject == hEndlimitToggletool
      set( hStartlimitToggletool, 'State', 'off' );
    else
      set( hEndlimitToggletool,   'State', 'off' );
    end
    
    % Activate clic mouse menu on first axes (salinity) for next rbbox
    % DO NOT USE :
    % ButtonDownFcn -  the callback executes whenever you press a button
    % while the pointer is within the axes, but not over another graphics
    % object parented to the axes.
    % -------------------------------------------------------------------
    %set(hPlotAxes(1),'ButtonDownFcn', @Time_SelectCallback);
    set( hMainFig,'WindowButtonDownFcn', @Time_SelectCallback);  
 
    % change cursor to crosshair aspect
    % ---------------------------------
    set( hMainFig, 'Pointer', 'custom',...
         'PointerShapeCData', selTimePointer, 'PointerShapeHotSpot',[9 9]);

    % ----------------------------------------------------------------------
    % nested function on mouse clic when Select Time toggle tool is selected
    % ----------------------------------------------------------------------
    function Time_SelectCallback(gcbo, eventdata)

      % disable ButtonMotion on main fig during select
      % prevent drawing to map
      % ----------------------------------------------
      %set( hMainFig, 'WindowButtonMotionFcn', []);

      % Get the current mouse position on axe 2
      % ---------------------------------------
      a = get(hPlotAxes(2), 'CurrentPoint');
      x = a(2,1);

      % Write the date in the Editable uicontrol
      % ----------------------------------------
      if hObject == hEndlimitToggletool
        set( hetDateMax, 'String', datestr(x, 31));
      else
        set( hetDateMin, 'String', datestr(x, 31));
      end

      % Color of date limit in red if dateMax <=datMin
      % -------------------------------------------------------------------
      dateMin = datenum(get( hetDateMin, 'String'), 'yyyy-mm-dd HH:MM:SS');
      dateMax = datenum(get( hetDateMax, 'String'), 'yyyy-mm-dd HH:MM:SS');

      if dateMax <= dateMin
        set( hetDateMin, 'ForegroundColor','r');
        set( hetDateMax, 'ForegroundColor','r');
      else
        set( hetDateMin, 'ForegroundColor','k');
        set( hetDateMax, 'ForegroundColor','k');
      end

      % Enable ButtonMotion on main fig 
      % -------------------------------
      %set( hMainFig, 'WindowButtonMotionFcn', @MouseMotion);
      
    end
  end

%% SelectTime_OffMenuCallback ........................... Correction module
%--------------------------------------------------------------------------
  function SelectTime_OffMenuCallback(hObject, eventdata)
  % Callback function run when the ....

    % Desactivate time limit buttons
    % ------------------------------
    if hObject == hEndlimitToggletool
      set( hEndlimitToggletool, 'State', 'off');
    else
      set( hStartlimitToggletool, 'State', 'off');
    end
    
    set( hMainFig,'WindowButtonDownFcn', []);  
    
    set(hMainFig,'Pointer','arrow');

  end

%% CorCancelCallback .................................... Correction Module
  function CorCancelCallback(hObject, eventdata)
    % Callback function run when ...

    % Desactivate somme Toggle button
    % -------------------------------
    set( hZoomToggletool,       'state', 'off' );
    set( hPanToggletool,        'state', 'off' );
    set( hQCToggletool,  'state', 'off' );
    set( hStartlimitToggletool, 'State', 'off' );
    set( hEndlimitToggletool,   'State', 'off' );

    tsg = getappdata(hMainFig, 'tsg_data');
    
    tsg.SSPS_ADJUSTED       = tsg.SSPS;
    tsg.SSPS_ADJUSTED_ERROR = NaN*ones(size(tsg.SSPS));
    tsg.SSPS_ADJUSTED_QC    = tsg.SSPS_QC;

    setappdata(hMainFig, 'tsg_data', tsg);

    % Reinitialise plot 1 and 3
    % -------------------------
    erase_Line( hPlotAxes, 1 )
    plot_Sample( hMainFig, hPlotAxes, 1, 'SSPS_DIF', [] );
    erase_Line( hPlotAxes, 3 )  
    plot_Tsg( hMainFig, hPlotAxes, 3, tsg.DAYD, tsg.SSPS, [], 'SSPS','k','none','.',1);

  end

%% CorLinearCallback .................................... Correction Module
  function CorLinearCallback(hObject, eventdata)
    % Callback function run when

    % Desactivate somme Toggle button
    % -------------------------------
    set( hZoomToggletool,       'state', 'off' );
    set( hPanToggletool,        'state', 'off' );
    set( hQCToggletool,  'state', 'off' );
    set( hStartlimitToggletool, 'State', 'off' );
    set( hEndlimitToggletool,   'State', 'off' );

    % Get the time limits for the correction A TESTER
    % --------------------------------------
    dateMin = datenum(get( hetDateMin, 'String'), 'yyyy-mm-dd HH:MM:SS');
    dateMax = datenum(get( hetDateMax, 'String'), 'yyyy-mm-dd HH:MM:SS');

    % Compute the sample-TSG differences
    % ----------------------------------
    diffTsgSample( hMainFig );

    % Correction
    % ----------
    error = corTsgLinear(hMainFig, dateMin, dateMax);

    switch error

      case 1

        % Plot in the 3 axes
        % ------------------
        plot_Correction( hMainFig, hPlotAxes );

      case -1
        msgbox( 'Date limits are not correct',...
          'Correction module', 'warn', 'modal');
    end

  end

%% CorMedianCallback .................................... Correction Module
  function CorMedianCallback(hObject, eventdata)
    % Callback function run when

    % Desactivate somme Toggle button
    % -------------------------------
    set( hZoomToggletool,       'state', 'off' );
    set( hPanToggletool,        'state', 'off' );
    set( hQCToggletool,  'state', 'off' );
    set( hStartlimitToggletool, 'State', 'off' );
    set( hEndlimitToggletool,   'State', 'off' );

    % Get the time limits for the correction A TESTER
    % --------------------------------------
    dateMin = datenum(get( hetDateMin, 'String'), 'yyyy-mm-dd HH:MM:SS');
    dateMax = datenum(get( hetDateMax, 'String'), 'yyyy-mm-dd HH:MM:SS');

    % Compute the sample-TSG differences
    % ----------------------------------
    diffTsgSample( hMainFig );

    % Correction
    % ----------
    error = corTsgMedian(hMainFig, dateMin, dateMax);

    switch error

      case 1

        % Plot in the 3 axes
        % ------------------
        plot_Correction( hMainFig, hPlotAxes );

      case -1
        msgbox( 'Date limits are not correct',...
          'Correction module', 'warn', 'modal');
    end

  end

%% Clim_OffMenuCallback
  %------------------------------------------------------------------------
  % Callback function run when the Levitus climatology toolbar is unselected
  %------------------------------------------------------------------------
  function Clim_OffMenuCallback(hObject, eventdata)

    % Get lines handles from tag
    % --------------------------
    hLines = findobj('-regexp', 'Tag', 'TAG_LINE_CLIMATO_');
    
    % Delete climatology lines on axes
    % ---------------------------------
    delete(hLines);
    
  end

%% Clim_OnMenuCallback
  %------------------------------------------------------------------------
  % Callback function run when the Levitus climatology toolbar is unselected
  %------------------------------------------------------------------------
  function Clim_OnMenuCallback(hObject, eventdata)

    % Test if the TSG and bucket files have been read
    % -----------------------------------------------
    if strcmp( get(hOpenMenu, 'UserData'), 'on' )

      % plot climatology
      % ----------------
      plot_Climatology(hMainFig, hPlotAxes);
      
    end
  end

%% ClimatoSelectMenuCallback
  % -------------------------------------------------------------------
  % Callback function run when climato submenu is selected
  % -------------------------------------------------------------------
  function ClimatoSelectMenuCallback(hObject, eventdata, climato, time)
    
    % find all climato submenu and set 'checked' property to 'off'
    % ------------------------------------------------------------
    hdl = findobj( '-regexp', 'tag', 'TAG_UIMENU_CLIMATO');
    set(hdl, 'checked', 'off');
    
    % set current climato submenu checked
    % -----------------------------------
    set(hObject, 'checked', 'on');
    
    % memorize action on climatology menu for next use
    % ------------------------------------------------
    s.type = climato;
    s.time = time;
    set(hClimatoMenu, 'userdata', s);
    
    % check if climatology toggle button is set
    % -----------------------------------------
    if strcmp(get(hClimToggletool, 'state'), 'on')
      
      % clear last plotted climatology
      % ------------------------------
      Clim_OffMenuCallback;
      
      % plot and read (eventually) new climatology
      % ------------------------------------------
      plot_Climatology(hMainFig, hPlotAxes);
    end
    
  end

%% PreferencesMenuCallback
  % -------------------------------------------------------------------
  % Callback function run when Option/Preference is selected
  % -------------------------------------------------------------------
  function PreferencesMenuCallback(hObject, eventdata)
    
    % Test if the preference menu is allready checked
    % -----------------------------------------------
    if strcmp( get(hObject, 'checked'), 'on' )
      
      % set preference autoload off (unchecked)
      % ---------------------------------------
      root.preference.autoload = 'off';
      
    else
      
      % set preference autoload on (checked)
      % ------------------------------------
      root.preference.autoload = 'on';
      
    end

     % set current option/preference submenu state
     % -------------------------------------------
     set(hObject, 'checked', root.preference.autoload);
     
    % store root struct to root UserData
    % ----------------------------------
    set(0, 'userdata', root);
    
  end


%% HeaderMenuCallback
% -------------------------------------------------------------------
% Callback function run when the headerForm tool bar item is selected
% -------------------------------------------------------------------
  function HeaderMenuCallback(hObject, eventdata)

    % call header form function
    % -------------------------
    headerForm(hMainFig);

  end

%% SaveMenuCallback
% -------------------------------------------------------------------
% Callback function run when the Save menu item is selected
% -------------------------------------------------------------------
  function SaveMenuCallback(hObject, eventdata)

    % Retrieve named application data
    % -------------------------------
    tsg = getappdata( hMainFig, 'tsg_data');

    % get fileName without extension in cell
    % --------------------------------------
    file = textscan( tsg.file.name,'%s','delimiter','.');

    % fill or append header form
    % -------------------------
    error = headerForm(hMainFig);

    % if user press continue button, ask for netcdf file
    % ------------------------------------------------
    if error ~= -1
      [fileName, pathName, filterIndex] = uiputfile('*.nc', ...
        'Save file name', [file{1}{1} '.nc']);

      % if user press cancel button, all var set to zero
      % ------------------------------------------------
      if fileName == 0
        return;
      end

      % Pointer set to watch during reading and plotting
      % ------------------------------------------------
      set( hMainFig, 'Pointer', 'watch' );

      % flushes the event queue and updates the closed uiputfile window
      % ---------------------------------------------------------------
      drawnow;

      % write netcdf file
      % -----------------
      error = writeTSGDataNetCDF( hMainFig, [pathName fileName] );

      % Pointer reset to arrow
      % ----------------------
      set( hMainFig, 'Pointer', 'arrow' );

      % Check for NetCDF writing error 
      % must to be rewriting
      % ------------------------------
      if error == -1
        warning('tsgqc_GUI:SaveMenuCallback', ...
                'NetCDF writing error: %s %s', pathName, fileName);
        return;
      end

      % update the display
      % ------------------
      set( hInfoFileText, 'String', fileName);

      % enable Quality Control mode
      % ---------------------------
      hdl_pushtool = findobj('Tag', 'QC');
      set(hdl_pushtool, 'Enable', 'on');
    end

  end

%% UndoMenuCallback
  % -----------------------------------------------------------------------
  % Callback function run when the Edit/Undo menu item is selected (Ctrl+Z)
  % -----------------------------------------------------------------------
  function UndoMenuCallback(hObject, eventdata)  
 
    % Undo module not yet implemented
    % -------------------------------
    % msgbox('Undo module not yet implemented', 'modal');
    
    tsg.queue = undo(tsg.queue);
    tsg.SSPS_QC = get(tsg.queue);
   
    % Make the Salinity, temperature and velocity plot
    % ------------------------------------------------
    plot_SalTempVel( hMainFig, hPlotAxes );

  end

%% RedoMenuCallback
  % -----------------------------------------------------------------------
  % Callback function run when the Edit/Redo menu item is selected (Ctrl+R)
  % -----------------------------------------------------------------------
  function RedoMenuCallback(hObject, eventdata)  
 
    % Redo module not yet implemented
    % -------------------------------
    msgbox('Redo module not yet implemented', 'modal');
    
  end

%% QuitMenuCallback
  % -----------------------------------------------------------------
  % Callback function run when the Quit menu item is selected
  % -----------------------------------------------------------------
  function QuitMenuCallback(hObject, eventdata)  

    % get root struct from root object
    % --------------------------------
    root = get(0,'Userdata');

    % save config mat file in prefdir
    % -------------------------------
    config_file = [prefdir, filesep, tsgqcname, '.mat'];

    % set tsg.levitus empty before save preferences in mat file
    % ---------------------------------------------------------
    tsg.levitus = [];

    % save preference mat file
    % ------------------------
    save( config_file, 'root', 'tsg');

    % If the data have been modified and not save, the program
    % ask to save the data
    % --------------------------------------------------------
    if  strcmp( get( hSaveMenu, 'UserData' ), 'on')
      selection = ...
        questdlg('The file has been modified.  Do you want to save it ?',...
        'Save before Quit?',...
        'Yes', 'No', 'Yes');
      
      if strcmp(selection, 'Yes')
        
        % call File/Save Menu Callback before before quit
        % -----------------------------------------------
        SaveMenuCallback;
             
      end
      
      % quit program
      % ------------
      QuitProgram;
      
    else
      selection = ...
        questdlg(['Quit ' get(hMainFig, 'Name') '?'],...
        ['Quit ' get(hMainFig, 'Name') '?'],...
        'Yes', 'No', 'Yes');
      if strcmp(selection, 'No')
        return;
      else
        QuitProgram;
      end
    end

  end

%% QuitProgram
  % -----------------------------------------------------------------
  % call from: 
  %   QuitMenuCallback
  %   callback 'CloseRequestFcn', @QuitProgram
  % -----------------------------------------------------------------
  function QuitProgram(hObject, eventdata)
    
    % close the main windows
    % ----------------------
    delete(hMainFig);
    
    % reset userdata property of root Matalab object (0) for next use
    % ---------------------------------------------------------------
    set(0, 'userdata', []);

    % reset Matlab search path to default
    % -----------------------------------
    rmpath( [DEFAULT_PATH_FILE filesep 'tsg_util'] );
    rmpath( [DEFAULT_PATH_FILE filesep 'tsg_data'] );
    rmpath( [DEFAULT_PATH_FILE filesep 'tsg_io'] );
    
    % Refresh file system caches
    % --------------------------
    rehash;
    
  end

% end of tsgqc_GUI
% ----------------
end  
